@@ -6,7 +6,7 @@
 #'  simulated by bootstrap and recorded. A 2-D spatial and temporal ETAS model is
 #'  fitted to each bootstrap-simulated
 #'  earthquake data catalog, and the corresponding parameter estimates are
-#'  recorded, which provides as many distributions of estimates.
+#'  recorded, which provides an empirical distribution for each estimate.
 #'  For a given confidence level \eqn{1-\alpha} (0.95 by default), bootstrap
 #'  confidence intervals are built from the empirical \eqn{\alpha/2} (0.025) and
 #'  \eqn{1 -\alpha/2} (0.975) quantiles
@@ -22,31 +22,32 @@
 #'  \deqn{\lambda_{\beta,\mathbf{\theta}}(t,x,y,m
 #'  \mid H_t) = s_{\beta}(m)\lambda_{\mathbf{\theta}}(t,x,y \mid H_t),}
 #'  where \eqn{\beta} and \eqn{\mathbf{\theta} = (\nu,A,\alpha,c,p,q,D,\gamma)}
-#'  are the model parameters. \eqn{s_\beta} is the probability density function
+#'  are the model parameters and \eqn{s_\beta} is the probability density function
 #'  (pdf) associated with the distribution of earthquake magnitudes. It is
 #'  assumed that the distribution of the magnitude of earthquakes is independent
-#'  of the distributions of the occurrence time of earthquakes and the 2-D
+#'  of the joint distribution of the occurrence time of earthquakes and the 2-D
 #'  spatial location of their epicenters. It can be expressed, for arbitrary
-#'  \eqn{\beta \in (0, \infty)} as \deqn{s_{\beta}(m) = \beta \exp \{
+#'  \eqn{\beta \in (0, \infty)}, as \deqn{s_{\beta}(m) = \beta \exp \{
 #'  -\beta(m-m_0)\},} where \eqn{m} and \eqn{m_0} represent the magnitude of the
 #'  earthquake and the magnitude threshold, respectively.
-#'  \eqn{\lambda_{\mathbf{\theta}}(t,x,y \mid H_t)} represents the rate of
+#'  Moreover, \eqn{\lambda_{\mathbf{\theta}}(t,x,y \mid H_t)} represents the rate of
 #'  observation of earthquakes in time and space, given the information on
 #'  earthquakes prior to time \eqn{t}. This rate is expressed as the sum of two
-#'  terms and is defined as
+#'  terms, namely
 #'  \deqn{\lambda_{\mathbf{\theta}}(t,x,y \mid H_t) = \mu(x,y) + \sum_{i:t_i<t}k(m_i)g(t-t_i)f(x-x_i,y-y_i \mid m_i)}
 #'   with
 #'  \deqn{\mu(x,y) = \nu u(x,y),}where \eqn{\nu \in (0, \infty)}.
-#'  The term \eqn{\mu(x,y)} is usually called ``background seismicity rate" and represents the rate at which earthquakes independently occur around longitude \eqn{x} and latitude \eqn{y}.
+#'  The term \eqn{\mu(x,y)} is usually called "background seismicity rate" and represents the rate at which earthquakes independently occur around longitude \eqn{x} and latitude \eqn{y}.
 #'  The \eqn{i}th term of the summation in \eqn{\lambda_{\theta}}, namely
-#'  \deqn{k(m_i)g(t-t_i)f(x-x_i,y-y_i \mid m_i)}
+#'  \deqn{k(m_i)g(t-t_i)f(x-x_i,y-y_i \mid m_i),}
 #'  represents the effect of the \eqn{i}th earthquake before time \eqn{t} on the occurrence rate of earthquakes that would occur at time \eqn{t}, with an epicenter around \eqn{(x,y)}. Thus,
 #'  \deqn{\sum_{i:t_i<t}k(m_i)g(t-t_i)f(x-x_i,y-y_i \mid m_i)} describes the total effect of all the earthquakes that occurred prior to time \eqn{t}, on the rate at which earthquakes would occur with an epicenter around \eqn{(x, y)} at time \eqn{t}.
-#'  The expressions of \eqn{k}, \eqn{g}, and \eqn{f} are discussed individually as follows. First,
+#'  The expressions for \eqn{k}, \eqn{g}, and \eqn{f} are discussed individually as follows. First,
 #'  \deqn{ k(m) = Ae^{\alpha(m-m_0)},\quad m \geq m_0 ,} can be interpreted as the expected number of earthquakes triggered by a previous earthquake with magnitude \eqn{m}, where \eqn{A \in (0, \infty)} and \eqn{\alpha \in (0, \infty)}. Second, for all \eqn{t \in (t_i, \infty)},
-#'  \deqn{g(t-t_i) = \frac{p-1}{c} \, \left (1+\frac{t-t_i}{c} \right )^{-p},} is the pdf for the occurrence time of an earthquake triggered by the \eqn{i}th earthquake in the catalog, which occurred at time \eqn{t_i}, where \eqn{c \in (0, \infty)} and \eqn{p \in (1, \infty)}. Third,
+#'  \deqn{g(t-t_i) = \frac{p-1}{c} \, \left (1+\frac{t-t_i}{c} \right )^{-p}} is the pdf for the occurrence time of an earthquake triggered by the \eqn{i}th earthquake in the catalog, which occurred at time \eqn{t_i}, where \eqn{c \in (0, \infty)} and \eqn{p \in (1, \infty)}. Third,
 #'  \deqn{f(x-x_i,y-y_i \mid m_i) = \frac{q-1}{\pi De^{\gamma(m_i-m_0)}} \, \left\{ 1+\frac{(x-x_i)^2+(y-y_i)^2}{De^{\gamma(m_i-m_0)}} \right\}^{-q}}
 #'  is the pdf for the occurrence location (epicenter) of an earthquake triggered by the \eqn{i}th earthquake in the catalog, which occurred with magnitude \eqn{m_i} and an epicenter at \eqn{(x_i, y_i)}, where \eqn{D \in (0, \infty)}, \eqn{\gamma \in (0, \infty)}, and \eqn{q \in (1, \infty)}.
+#'  For more details, see the articles of Zhuang et al. (2002, 2004). 
 #'
 #'@param earthquake_data
 #' An object of class "data.frame" containing the following 5 columns:
@@ -57,31 +58,26 @@
 #' \item latitude: Latitude of the epicenter of earthquakes in decimal degrees
 #' \item magnitude: Magnitude of earthquakes (Any type of magnitude is accepted as far as it is used consistently and thoroughly.)
 #'}
-#'See VCI_earthquakes for an example.
+#'See VCI_earthquakes for an example with a rectangular study region; for a more general, polygonal study region, see JPN_earthquakes.
 #'Note that West longitude and South latitude
-#'  values should be negative, whereas the East longitude and North latitude
+#'  values should be negative, whereas East longitude and North latitude
 #'  values are positive.
 #'@param longitude_boundaries A numerical vector of length 2 (long_min, long_max)
 #'  with the longitude boundaries of a rectangular space window,
 #'  for which the earthquake catalog data are contained in \bold{earthquake_data}. If NULL (at the beginning of the execution of the program),
 #'  long_min and long_max will be set (by the program) to the minimum and maximum values of the longitudes
-#'  of earthquakes in \bold{earthquake_data}.
+#'  of earthquakes in \bold{earthquake_data}. 
+#'  Together with \bold{latitude_boundaries}, \bold{longitude_boundaries} defines a region aimed to take edge effects into account in the analyses. 
+#'  This region includes the study region and approximately 20\% more space around the study region.
 #'@param latitude_boundaries A numerical vector of length 2 (lat_min, lat_max)
 #'  with the latitude boundaries of a rectangular space window, for which
 #'  the earthquake catalog data are contained in \bold{earthquake_data}. If NULL, lat_min and lat_max will be set to the minimum and maximum values of the latitudes of
 #'  earthquakes in \bold{earthquake_data}.
-#'@param longitude_study A numerical vector of length 2 (long_study_min,
-#'  long_study_max) with the longitude boundaries of a rectangular "study space
-#'  window"
-#'@param latitude_study A numerical vector of length 2 (lat_study_min,
-#'  lat_study_max) with the latitude boundaries of the study space
-#'  window.
-#'  Note: The study space window should be narrower than the
-#'  rectangular space window defined by \bold{longitude_boundaries} and
-#'  \bold{latitude_boundaries}. If NULL, \bold{longitude_study} and \bold{latitude_study} will be
-#'  filled with boundaries defining a rectangular space window 20\% narrower than the space window built from the \bold{longitude_boundaries} and \bold{latitude_boundaries}, while keeping the same center.
+#'@param study_region A list with two components (lat, long) of equal length specifying the coordinates
+#'  of the vertices of a polygonal study region. The vertices must be written in anticlockwise order. If NULL, study_region will
+#'  be filled with boundaries defining a rectangular space window 20\% narrower than the space window built from the longitude_boundaries and latitude_boundaries, while keeping the same center.
 #'@param time_begin A character string, in the date-time format (yyyy-mm-dd
-#'  hh:mm:ss), which identifies the beginning of the time span in
+#'  hh:mm:ss), which identifies the start of the time span in
 #'  \bold{earthquake_data}. If NULL, \bold{time_begin} will be set to the date-time of the
 #'  first event in \bold{earthquake_data}.
 #'
@@ -94,27 +90,27 @@
 #'  \bold{earthquake_data}.
 #'  Note: \bold{study_end} coincides with the end of the time span in
 #'  \bold{earthquake_data}.
-#'@param magnitude_threshold A decimal number, which specifies the threshold to be used for the
-#'  magnitudes of earthquakes. Only earthquakes with a magnitude greater or
+#'@param magnitude_threshold A decimal number which specifies the threshold to be used for the
+#'  magnitudes of earthquakes. Only earthquakes with a magnitude greater than or
 #'  equal to \bold{magnitude_threshold} will be considered, while the model
-#'  is fitting.
-#'  is used consistently and thoroughly. If NULL, the minimum magnitude
+#'  is being fitted. If NULL, the minimum magnitude
 #'  calculated from the events in \bold{earthquake_data} will be used for
 #'  \bold{magnitude_threshold}.
 #'
 #'@param time_zone A character string specifying the time zone in
 #'  which the occurrence times of earthquakes were recorded.
 #'  The default "GMT"is the UTC (Universal Time Coordinates).
-#'@param parameters_0 A decimal vector of size 8 \eqn{(\nu, A, c, \alpha, p, D, q, \gamma)}, to be used as an initial solution for the
+#'@param round_off A logical flag indicating whether or not to account for round-off error in coordinates of epicenters. 
+#'@param parameters_0 A decimal vector of size 8 \eqn{(\nu, A, c, \alpha, p, D, q, \gamma)} to be used as an initial solution for the
 #' iterative maximum likelihood estimation of the ETAS model parameters.
 #' In particular, the values of parameters \eqn{\nu},
 #'  \eqn{A}, \eqn{c}, \eqn{\alpha}, \eqn{D}, and \eqn{\gamma} are positive,
 #'  and those of
-#'  \eqn{p} and \eqn{q} are greater than 1.
+#'  \eqn{p} and \eqn{q} are strictly greater than 1.
 #'  If NULL, the values recommended by Ogata (1998) will be used.
-#'@param number_simulations A positive integer, which stands for the number of
+#'@param number_simulations A positive integer which stands for the number of
 #'  requested bootstrap simulations. The default value is 1000.
-#'@param confidence_level A decimal number in (0, 1), which specifies the
+#'@param confidence_level A decimal number in (0, 1) which specifies the
 #'  confidence level associated with the bootstrap confidence intervals that are built for
 #'  the ETAS model parameters, and saved as outputs.
 #'  It is set to 0.95 by default.
@@ -148,9 +144,10 @@
 #'@export
 #'
 #'@references
-#'  Jalilian, A. (2019). ETAS: An R package for fitting the space-time ETAS
-#'  model to earthquake data. Journal of Statistical Software 88, 1–39.
-#'
+#'  Dutilleul, P., Genest, C., Peng, R., 2024. Bootstrapping for parameter uncertainty
+#'  in the space-time epidemic-type aftershock sequence model. Geophysical Journal 
+#'  International 236, 1601–1608.
+#'  
 #'  Ogata, Y. (1998). Space-time point-process models for earthquake
 #'  occurrences. Annals of the Institute of Statistical Mathematics 50(2),
 #'  379–402.
@@ -169,8 +166,8 @@
 #'ETAS_Boots(earthquake_data = VCI_earthquakes,
 #'           longitude_boundaries = c(-131, -126.25),
 #'           latitude_boundaries = c(48, 50),
-#'           longitude_study = c(-130.5,-126.75),
-#'           latitude_study = c(48.25,49.75),
+#'           study_region = list(long = c(-130.5, -130.5, -126.75, -126.75),
+#'                               lat = c(49.75, 48.25, 48.25, 49.75)),
 #'           time_begin = "2000/01/01 00:00:00",
 #'           study_start = "2008/04/27 00:00:00",
 #'           study_end = "2018/04/27 00:00:00",
@@ -181,17 +178,40 @@
 #'           confidence_level = 0.95,
 #'           output_datasets = FALSE,
 #'           output_estimates = FALSE)}
+#'           
+#'@examples\donttest{
+#'ETAS_Boots(
+#'  earthquake_data=JPN_earthquakes,
+#'  longitude_boundaries = c(128, 145),
+#'  latitude_boundaries = c(27, 45),
+#'  study_region = list(long=c(130,135,145,140),
+#'                     lat=c(33,30,40,43)),
+#'  time_begin = "1926-01-08",
+#'  study_start = "1953-05-26",
+#'  study_end = "1990-01-08",
+#'  magnitude_threshold = 5.5,
+#'  time_zone = "GMT",
+#'  round_off = FALSE,
+#'  parameters_0 = c(0.524813924, 0.09, 0.045215442, 1.970176559, 
+#'                   1.249620329, 0.002110203, 1.910492169,1.763149113 ),
+#'  number_simulations = 2,
+#'  confidence_level = 0.95,
+#'  output_datasets = FALSE,
+#'  output_estimates = FALSE
+#')
+#'
+#'}           
 
 ETAS_Boots<- function(earthquake_data,
                      longitude_boundaries=NULL,
                      latitude_boundaries=NULL,
-                     longitude_study=NULL,
-                     latitude_study=NULL,
+                     study_region = NULL, 
                      time_begin=NULL,
                      study_start=NULL,
                      study_end=NULL,
                      magnitude_threshold=NULL,
                      time_zone="GMT",
+                     round_off = FALSE,
                      parameters_0=NULL,
                      number_simulations=1000,
                      confidence_level=0.95,
@@ -201,8 +221,7 @@ ETAS_Boots<- function(earthquake_data,
   E_data<- as.data.frame(earthquake_data)
   long_range<- longitude_boundaries
   lat_range<- latitude_boundaries
-  long_study<- longitude_study
-  lat_study<- latitude_study
+  study_region<- study_region
   t_begin<- time_begin
   s_start<- study_start
   s_end<- study_end
@@ -239,21 +258,15 @@ ETAS_Boots<- function(earthquake_data,
   m <- E_data[,5]   # magnitude of earthquakes
 
   #check the argument related to spatial region
-
+  
   if (is.null(long_range)){
     long_range<- c(min(x),max(x))
-  } else if (!is.vector(long_range)
-             || length(long_range) != 2
-             || long_range[2] <= long_range[1])
-  {stop("longitude_range must be a vector of length 2 giving (long_min, long_max)")}
-
+  } 
+  
   if (is.null(lat_range)){
     lat_range<- c(min(y),max(y))
-  } else if (!is.vector(lat_range)
-             || length(lat_range) != 2
-             || lat_range[2] <= lat_range[1])
-  {stop("latitude_range must be a vector of length 2 giving (lat_min, lat_max)")}
-
+  }
+  
   if (min(x) <long_range[1]
       ||max(x) >long_range[2]
       ||min(y) <lat_range[1]
@@ -261,36 +274,42 @@ ETAS_Boots<- function(earthquake_data,
     stop("at least one earthquake in the given earthquake_data with epicenter outside
     the rectangular window constructed by longitude_rang and latitude_range ")
   }
-
-  if ((is.null(long_study) & !is.null(lat_study))
-      || (is.null(lat_study) & !is.null(long_study)))
-    stop("only one of longitude_study and latitude_study is set by the user")
-
-  if (is.null(long_study)){
-    long_study<- sum(long_range)/2+c(-1,1)*sqrt(0.8)*(long_range[2]-long_range[1])/2
-  } else if (!is.vector(long_study)
-             || length(long_study) != 2
-             || long_study[2] <= long_study[1])
-    {
-    stop("longitude_study must be a vector of length 2 giving (long_study_min, long_study_max)")
-  } else if (long_study[1]<=long_range[1]
-             || long_study[2]>=long_range[2])
-  {stop("study space window built by longitude_study and latitude_study
-         should be narrower than space window constructed by
-         longitude_boundaries and latitude_boundaries.")}
-
-  if (is.null(lat_study)){
-    lat_study <- sum(lat_range)/2+c(-1,1)*sqrt(0.8)*(lat_range[2]-lat_range[1])/2
-  } else if (!is.vector(lat_study)
-             || length(lat_study) != 2
-             || lat_study[2] <= lat_study[1]){
-    stop("lat_study must be a vector of length 2 giving (lat_study_min, lat_study_max)")
-  }else if (lat_study[1]<= lat_range[1]
-            || lat_study[2]>= lat_range[2])
-  {stop("study space window built by longitude_study and latitude_study
-         should be narrower than the space window constructed by
-         longitude_boundaries and latitude_boundaries.")}
-
+  
+  if (is.null(study_region)){
+    long_study_min <- stats::median(long_range) - 0.5*(long_range[2]-long_range[1])*0.9
+    long_study_max <- stats::median(long_range) + 0.5*(long_range[2]-long_range[1])*0.9
+    lat_study_min <- stats::median(lat_range) - 0.5*(lat_range[2]-lat_range[1])*0.9
+    lat_study_max <- stats::median(lat_range) + 0.5*(lat_range[2]-lat_range[1])*0.9
+    study_region <- list(long = c(long_study_min, long_study_max, long_study_max, long_study_min),
+                        lat = c(lat_study_min, lat_study_min, lat_study_max, lat_study_max))
+  }
+  
+  if(setequal(names(study_region),c("lat","long")) ==FALSE){
+    stop("study_region would be a list with two components named lat and long ")
+  }
+  
+  if(length(study_region$long)!=length(study_region$lat)){
+    stop("lat and long must have equal length")
+  }
+  
+  if (!is.list(study_region)) {
+    stop("study_region would be a list with components lat and long of equal length
+         specifying the coordinates of the vertices of a polygonal study region. 
+         The vertices must be listed in anticlockwise order.")
+  }
+  
+  long_min<- min(study_region$long)
+  long_max<- max(study_region$long)
+  lat_min<- min(study_region$lat)
+  lat_max<-max(study_region$lat)
+  
+  if(long_min < min(long_range) |
+     long_max > max(long_range) |
+     lat_min < min(lat_range) |
+     lat_max > max(lat_range)) {
+    stop("The study space window must be inside the boundaries constructed by longitude_boundaries and latitude_boundaries. ")
+  }
+  
   # check the format of arguments related to time
   # extract date and time of events
   if (is.character(t_begin)!=TRUE){
@@ -363,19 +382,22 @@ ETAS_Boots<- function(earthquake_data,
   lat<- E_data[,4]
   mag<- E_data[,5]
   E_data<- data.frame(date,time,long,lat,mag)
-
-  S_total_cat_3.5<- ETAS::catalog(E_data,
+  
+  region.win <- spatstat.geom::owin(poly = list(x = study_region$long, 
+                                                y = study_region$lat))
+  
+  S_total_cat<- ETAS::catalog(E_data,
                                   time.begin= t_begin,
                                   study.start= s_start,
                                   study.end= s_end,
-                                  long.range = long_study,
-                                  lat.range =lat_study,
+                                  region.poly = study_region,
                                   mag.threshold=m_threshold,
-                                  tz=tz)
+                                  tz=tz,
+                                  roundoff = round_off)
 
 
 ## ------------------------------------------------------------------------------------------------
-  S.fit<- ETAS::etas(S_total_cat_3.5, param0 =param)
+  S.fit<- ETAS::etas(S_total_cat, param0 =param)
 
 ## ------------------------------------------------------------------------------------------------
 #sample distribution of magnitudes
@@ -384,12 +406,10 @@ ETAS_Boots<- function(earthquake_data,
   t_min<- ETAS::date2day(s_start, start= t_begin, tz=tz)
   t_max<- ETAS::date2day(s_end, start= t_begin, tz=tz)
   E_data_new<- E_data[t>=t_min& t<=t_max,]
+  
   mag_sample<- subset(E_data_new,
                       mag >= m_threshold
-                      &long >= long_study[1]
-                      &long <= long_study[2]
-                      &lat >= lat_study[1]
-                      &lat <= lat_study[2])$mag
+                      &spatstat.geom::inside.owin(x=long,y=lat, w=region.win))$mag
 
 
 ## ------------------------------------------------------------------------------------------------
@@ -410,11 +430,11 @@ ETAS_Boots<- function(earthquake_data,
 
       quakes<-c()
 
-      quakes<- data.frame(substr(S_total_cat_3.5$longlat.coord$dt,1,10),
-                 substr(S_total_cat_3.5$longlat.coord$dt,12,19),
-                 as.numeric(S_total_cat_3.5$longlat.coord[,1]),
-                 as.numeric(S_total_cat_3.5$longlat.coord[,2]),
-                 as.numeric(S_total_cat_3.5$revents[,4]+m_threshold),
+      quakes<- data.frame(substr(S_total_cat$longlat.coord$dt,1,10),
+                 substr(S_total_cat$longlat.coord$dt,12,19),
+                 as.numeric(S_total_cat$longlat.coord[,1]),
+                 as.numeric(S_total_cat$longlat.coord[,2]),
+                 as.numeric(S_total_cat$revents[,4]+m_threshold),
                  as.numeric(S.fit$bwd),
                  as.numeric(S.fit$pb))
 
@@ -462,12 +482,10 @@ ETAS_Boots<- function(earthquake_data,
                                       time.begin= t_begin,
                                       study.start= s_start,
                                       study.end= s_end,
-                                      long.range= long_study,
-                                      lat.range= lat_study,
-                                      mag.threshold=m_threshold)
+                                      region.poly = study_region,
+                                      mag.threshold=m_threshold,
+                                      roundoff = round_off)
 
-    #print(S_total_cat_sim)
-    #plot(S_total_cat_sim)
 
 
 
